﻿using BlackcmithWorkshopFileImplement;
using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDataModels.Models;
using BlacksmithWorkshopFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopFileImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataFileSingleton _source;
        public ShopStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        public List<ShopViewModel> GetFullList()
        {
            return _source.Shops
            .Select(x => x.GetViewModel)
            .ToList();
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel
        model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            return _source.Shops
            .Where(x => x.ShopName.Contains(model.ShopName))
            .Select(x => x.GetViewModel)
            .ToList(); ;
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            return _source.Shops
            .FirstOrDefault(x => (!string.IsNullOrEmpty(model.ShopName) && x.ShopName ==
            model.ShopName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            model.Id = _source.Shops.Count > 0 ? _source.Shops.Max(x =>
            x.Id) + 1 : 1;
            var newShop = Shop.Create(model);
            if (newShop == null)
            {
                return null;
            }
            _source.Shops.Add(newShop);
            _source.SaveShops();
            return newShop.GetViewModel;
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            var component = _source.Shops.FirstOrDefault(x => x.Id ==
            model.Id);
            if (component == null)
            {
                return null;
            }
            component.Update(model);
            _source.SaveShops();
            return component.GetViewModel;
        }
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            var element = _source.Shops.FirstOrDefault(x => x.Id ==
            model.Id);
            if (element != null)
            {
                _source.Shops.Remove(element);
                _source.SaveShops();
                return element.GetViewModel;
            }
            return null;
        }
        private bool CheckSell(int ManufactureId, int count)
        {
            count -= _source.Shops.Select(x => x.ShopManufactures.Select(y => 
            (y.Value.Item1.Id == ManufactureId ? y.Value.Item2 : 0)).Sum()).Sum();
            return count <= 0;
        }
        public bool SellManufactures(IManufactureModel model, int count)
        {
            var neededManufacture = _source.Manufactures.FirstOrDefault(x => x.Id == model.Id);
            if (neededManufacture == null || !CheckSell(neededManufacture.Id, count))
            {
                return false;
            }
            for (int i = 0; i < _source.Shops.Count; i++)
            {
                var shop = _source.Shops[i];
                var assortment = shop.ShopManufactures;
                foreach (var manufacture in assortment.Where(x => x.Value.Item1.Id == neededManufacture.Id))
                {
                    var min = Math.Min(manufacture.Value.Item2, count);
                    assortment[manufacture.Value.Item1.Id] = (manufacture.Value.Item1, manufacture.Value.Item2 - min);
                    count -= min;
                    if (count <= 0)
                    {
                        break;
                    }
                }
                shop.Update(new ShopBindingModel
                {
                    Id = shop.Id,
                    ShopName = shop.ShopName,
                    Address = shop.Address,
                    OpeningDate = shop.OpeningDate,
                    MaxCapacity = shop.MaxCapacity,
                    ShopManufactures = assortment
                });
            }
            _source.SaveShops();
            return true;
        }
    }
}
