﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDatabaseImplement.Models;
using BlacksmithWorkshopDataModels.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BlacksmithWorkshopDatabaseImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        public ShopViewModel? Delete(ShopBindingModel model)
        {
            using var context = new BlacksmithWorkshopDataBase();
            var element = context.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (element != null)
            {
                context.Shops.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
        public ShopViewModel? GetElement(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName) && !model.Id.HasValue)
            {
                return null;
            }
            using var context = new BlacksmithWorkshopDataBase();
            return context.Shops
                .Include(x => x.Manufactures)
                .ThenInclude(x => x.Manufacture)
                .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)
                ?.GetViewModel;
        }
        public List<ShopViewModel> GetFilteredList(ShopSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ShopName))
            {
                return new();
            }
            using var context = new BlacksmithWorkshopDataBase();
            return context.Shops
                .Include(x => x.Manufactures)
                .ThenInclude(x => x.Manufacture)
                .Select(x => x.GetViewModel)
                .Where(x => x.ShopName.Contains(model.ShopName ?? string.Empty))
                .ToList();
        }
        public List<ShopViewModel> GetFullList()
        {
            using var context = new BlacksmithWorkshopDataBase();
            return context.Shops
                .Include(x => x.Manufactures)
                .ThenInclude(x => x.Manufacture)
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public ShopViewModel? Insert(ShopBindingModel model)
        {
            using var context = new BlacksmithWorkshopDataBase();
            try
            {
                var newShop = Shop.Create(context, model);
                if (newShop == null)
                {
                    return null;
                }
                if (context.Shops.Any(x => x.ShopName == newShop.ShopName))
                {
                    throw new Exception("Не должно быть два магазина с одним названием");
                }
                context.Shops.Add(newShop);
                context.SaveChanges();
                return newShop.GetViewModel;
            }
            catch
            {
                throw;
            }
        }
        public ShopViewModel? Update(ShopBindingModel model)
        {
            using var context = new BlacksmithWorkshopDataBase();
            var shop = context.Shops.FirstOrDefault(x => x.Id == model.Id);
            if (shop == null)
            {
                return null;
            }
            try
            {
                if (context.Shops.Any(x => (x.ShopName == model.ShopName && x.Id != model.Id)))
                {
                    throw new Exception("Не должно быть два магазина с одним названием");
                }
                shop.Update(model);
                shop.UpdateManufactures(context, model);
                context.SaveChanges();
                return shop.GetViewModel;
            }
            catch
            {
                throw;
            }
        }
        public bool SellManufactures(IManufactureModel model, int count)
        {
            if (model == null)
                return false;
            using var context = new BlacksmithWorkshopDataBase();
            using var transaction = context.Database.BeginTransaction();
            List<ShopManufacture> lst = new List<ShopManufacture>();
            foreach (var el in context.ShopManufactures.Where(x => x.ManufactureId == model.Id))
            {
                int dif = count;
                if (el.Count < dif)
                    dif = el.Count;
                el.Count -= dif;
                count -= dif;
                if (el.Count == 0)
                {
                    lst.Add(el);
                }
                if (count == 0)
                    break;
            }
            if (count > 0)
            {
                transaction.Rollback();
                return false;
            }
            foreach (var el in lst)
            {
                context.ShopManufactures.Remove(el);
            }
            context.SaveChanges();
            transaction.Commit();
            return true;
        }
    }
}
